/*
 * Test for ANSI Standard C: 7.10.1.1
 * 
 * #include <stdlib.h>
 * double atof (const char *nptr);
 * 
 * Copyright (c) 1996, Chris Nettleton Software
 *
 * The authors hereby grant permission to use, copy, modify, distribute, 
 * and license this software and its documentation for any purpose, 
 * provided that existing copyright notices are retained in all copies 
 * and that this notice is included verbatim in any distributions. No 
 * written agreement, license, or royalty fee is required for any of the 
 * authorized uses. Modifications to this software may be copyrighted by 
 * their authors and need not follow the licensing terms described here, 
 * provided that the new terms are clearly indicated on the first page 
 * of each file where they apply. 
 *
 * $Log$
 */

#include <report.h>

#include <stdlib.h>

#include <math.h>

static char *_test_atof_id =
  "$Id$";

void do_test ()
{
  /* Case 1 */
  if (!(atof ("0") == 0.0)) {
    failed ("Case 1: atof (\"0\") == 0.0");
  }
  
  /* Case 2 */
  if (!(atof ("0e10") == 0.0)) {
    failed ("Case 2: atof (\"0e10\") == 0.0");
  }
  
  /* Case 3 */
  if (!(atof ("0e-10") == 0.0)) {
    failed ("Case 3: atof (\"0e-10\") == 0.0");
  }
  
  /* Case 4 */
  if (!(atof ("-0") == 0.0)) {
    failed ("Case 4: atof (\"-0\") == 0.0");
  }
  
  /* Case 5 */
  if (!(atof ("0.0") == 0.0)) {
    failed ("Case 5: atof (\"0.0\") == 0.0");
  }
  
  /* Case 6 */
  if (!(atof ("-0.0") == 0.0)) {
    failed ("Case 6: atof (\"-0.0\") == 0.0");
  }
  
  /* Case 7 */
  if (!(atof ("1.0") == 1.0)) {
    failed ("Case 7: atof (\"1.0\") == 1.0");
  }
  
  /* Case 8 */
  if (!(atof ("2.0") == 2.0)) {
    failed ("Case 8: atof (\"2.0\") == 2.0");
  }
  
  /* Case 9 */
  if (!(atof ("3.0") == 3.0)) {
    failed ("Case 9: atof (\"3.0\") == 3.0");
  }
  
  /* Case 10 */
  if (!(atof ("4.0") == 4.0)) {
    failed ("Case 10: atof (\"4.0\") == 4.0");
  }
  
  /* Case 11 */
  if (!(atof ("5.0") == 5.0)) {
    failed ("Case 11: atof (\"5.0\") == 5.0");
  }
  
  /* Case 12 */
  if (!(atof ("6.0") == 6.0)) {
    failed ("Case 12: atof (\"6.0\") == 6.0");
  }
  
  /* Case 13 */
  if (!(atof ("7.0") == 7.0)) {
    failed ("Case 13: atof (\"7.0\") == 7.0");
  }
  
  /* Case 14 */
  if (!(atof ("8.0") == 8.0)) {
    failed ("Case 14: atof (\"8.0\") == 8.0");
  }
  
  /* Case 15 */
  if (!(atof ("9.0") == 9.0)) {
    failed ("Case 15: atof (\"9.0\") == 9.0");
  }
  
  /* Case 16 */
  if (!(atof ("-1.0") == -1.0)) {
    failed ("Case 16: atof (\"-1.0\") == -1.0");
  }
  
  /* Case 17 */
  if (!(atof ("-2.0") == -2.0)) {
    failed ("Case 17: atof (\"-2.0\") == -2.0");
  }
  
  /* Case 18 */
  if (!(atof ("-3.0") == -3.0)) {
    failed ("Case 18: atof (\"-3.0\") == -3.0");
  }
  
  /* Case 19 */
  if (!(atof ("-4.0") == -4.0)) {
    failed ("Case 19: atof (\"-4.0\") == -4.0");
  }
  
  /* Case 20 */
  if (!(atof ("-5.0") == -5.0)) {
    failed ("Case 20: atof (\"-5.0\") == -5.0");
  }
  
  /* Case 21 */
  if (!(atof ("-6.0") == -6.0)) {
    failed ("Case 21: atof (\"-6.0\") == -6.0");
  }
  
  /* Case 22 */
  if (!(atof ("-7.0") == -7.0)) {
    failed ("Case 22: atof (\"-7.0\") == -7.0");
  }
  
  /* Case 23 */
  if (!(atof ("-8.0") == -8.0)) {
    failed ("Case 23: atof (\"-8.0\") == -8.0");
  }
  
  /* Case 24 */
  if (!(atof ("-9.0") == -9.0)) {
    failed ("Case 24: atof (\"-9.0\") == -9.0");
  }
  
  /* Case 25 */
  if (!(atof ("1.0e1") == 1.0e1)) {
    failed ("Case 25: atof (\"1.0e1\") == 1.0e1");
  }
  
  /* Case 26 */
  if (!(atof ("2.0e2") == 2.0e2)) {
    failed ("Case 26: atof (\"2.0e2\") == 2.0e2");
  }
  
  /* Case 27 */
  if (!(atof ("3.0e3") == 3.0e3)) {
    failed ("Case 27: atof (\"3.0e3\") == 3.0e3");
  }
  
  /* Case 28 */
  if (!(atof ("4.0e4") == 4.0e4)) {
    failed ("Case 28: atof (\"4.0e4\") == 4.0e4");
  }
  
  /* Case 29 */
  if (!(atof ("5.0e5") == 5.0e5)) {
    failed ("Case 29: atof (\"5.0e5\") == 5.0e5");
  }
  
  /* Case 30 */
  if (!(atof ("6.0e6") == 6.0e6)) {
    failed ("Case 30: atof (\"6.0e6\") == 6.0e6");
  }
  
  /* Case 31 */
  if (!(atof ("7.0e7") == 7.0e7)) {
    failed ("Case 31: atof (\"7.0e7\") == 7.0e7");
  }
  
  /* Case 32 */
  if (!(atof ("8.0e8") == 8.0e8)) {
    failed ("Case 32: atof (\"8.0e8\") == 8.0e8");
  }
  
  /* Case 33 */
  if (!(atof ("9.0e9") == 9.0e9)) {
    failed ("Case 33: atof (\"9.0e9\") == 9.0e9");
  }
  
  /* Case 34 */
  if (!((fabs (atof ("0.00000000000000000000000000000000000001") / 1.0e-38 - 1.0) < 1.0e-10))) {
    failed ("Case 34: (fabs (atof (\"0.00000000000000000000000000000000000001\") / 1.0e-38 - 1.0) < 1.0e-10)");
  }
  
  /* Case 35 */
  if (!((fabs (atof ("0.0000000000000000000000000000000000001") / 1.0e-37 - 1.0) < 1.0e-10))) {
    failed ("Case 35: (fabs (atof (\"0.0000000000000000000000000000000000001\") / 1.0e-37 - 1.0) < 1.0e-10)");
  }
  
  /* Case 36 */
  if (!((fabs (atof ("0.000000000000000000000000000000000001") / 1.0e-36 - 1.0) < 1.0e-10))) {
    failed ("Case 36: (fabs (atof (\"0.000000000000000000000000000000000001\") / 1.0e-36 - 1.0) < 1.0e-10)");
  }
  
  /* Case 37 */
  if (!((fabs (atof ("0.00000000000000000000000000000000001") / 1.0e-35 - 1.0) < 1.0e-10))) {
    failed ("Case 37: (fabs (atof (\"0.00000000000000000000000000000000001\") / 1.0e-35 - 1.0) < 1.0e-10)");
  }
  
  /* Case 38 */
  if (!((fabs (atof ("0.0000000000000000000000000000000001") / 1.0e-34 - 1.0) < 1.0e-10))) {
    failed ("Case 38: (fabs (atof (\"0.0000000000000000000000000000000001\") / 1.0e-34 - 1.0) < 1.0e-10)");
  }
  
  /* Case 39 */
  if (!((fabs (atof ("0.000000000000000000000000000000001") / 1.0e-33 - 1.0) < 1.0e-10))) {
    failed ("Case 39: (fabs (atof (\"0.000000000000000000000000000000001\") / 1.0e-33 - 1.0) < 1.0e-10)");
  }
  
  /* Case 40 */
  if (!((fabs (atof ("0.00000000000000000000000000000001") / 1.0e-32 - 1.0) < 1.0e-10))) {
    failed ("Case 40: (fabs (atof (\"0.00000000000000000000000000000001\") / 1.0e-32 - 1.0) < 1.0e-10)");
  }
  
  /* Case 41 */
  if (!((fabs (atof ("0.0000000000000000000000000000001") / 1.0e-31 - 1.0) < 1.0e-10))) {
    failed ("Case 41: (fabs (atof (\"0.0000000000000000000000000000001\") / 1.0e-31 - 1.0) < 1.0e-10)");
  }
  
  /* Case 42 */
  if (!((fabs (atof ("0.000000000000000000000000000001") / 1.0e-30 - 1.0) < 1.0e-10))) {
    failed ("Case 42: (fabs (atof (\"0.000000000000000000000000000001\") / 1.0e-30 - 1.0) < 1.0e-10)");
  }
  
  /* Case 43 */
  if (!((fabs (atof ("0.00000000000000000000000000001") / 1.0e-29 - 1.0) < 1.0e-10))) {
    failed ("Case 43: (fabs (atof (\"0.00000000000000000000000000001\") / 1.0e-29 - 1.0) < 1.0e-10)");
  }
  
  /* Case 44 */
  if (!((fabs (atof ("0.0000000000000000000000000001") / 1.0e-28 - 1.0) < 1.0e-10))) {
    failed ("Case 44: (fabs (atof (\"0.0000000000000000000000000001\") / 1.0e-28 - 1.0) < 1.0e-10)");
  }
  
  /* Case 45 */
  if (!((fabs (atof ("0.000000000000000000000000001") / 1.0e-27 - 1.0) < 1.0e-10))) {
    failed ("Case 45: (fabs (atof (\"0.000000000000000000000000001\") / 1.0e-27 - 1.0) < 1.0e-10)");
  }
  
  /* Case 46 */
  if (!((fabs (atof ("0.00000000000000000000000001") / 1.0e-26 - 1.0) < 1.0e-10))) {
    failed ("Case 46: (fabs (atof (\"0.00000000000000000000000001\") / 1.0e-26 - 1.0) < 1.0e-10)");
  }
  
  /* Case 47 */
  if (!((fabs (atof ("0.0000000000000000000000001") / 1.0e-25 - 1.0) < 1.0e-10))) {
    failed ("Case 47: (fabs (atof (\"0.0000000000000000000000001\") / 1.0e-25 - 1.0) < 1.0e-10)");
  }
  
  /* Case 48 */
  if (!((fabs (atof ("0.000000000000000000000001") / 1.0e-24 - 1.0) < 1.0e-10))) {
    failed ("Case 48: (fabs (atof (\"0.000000000000000000000001\") / 1.0e-24 - 1.0) < 1.0e-10)");
  }
  
  /* Case 49 */
  if (!((fabs (atof ("0.00000000000000000000001") / 1.0e-23 - 1.0) < 1.0e-10))) {
    failed ("Case 49: (fabs (atof (\"0.00000000000000000000001\") / 1.0e-23 - 1.0) < 1.0e-10)");
  }
  
  /* Case 50 */
  if (!((fabs (atof ("0.0000000000000000000001") / 1.0e-22 - 1.0) < 1.0e-10))) {
    failed ("Case 50: (fabs (atof (\"0.0000000000000000000001\") / 1.0e-22 - 1.0) < 1.0e-10)");
  }
  
  /* Case 51 */
  if (!((fabs (atof ("0.000000000000000000001") / 1.0e-21 - 1.0) < 1.0e-10))) {
    failed ("Case 51: (fabs (atof (\"0.000000000000000000001\") / 1.0e-21 - 1.0) < 1.0e-10)");
  }
  
  /* Case 52 */
  if (!((fabs (atof ("0.00000000000000000001") / 1.0e-20 - 1.0) < 1.0e-10))) {
    failed ("Case 52: (fabs (atof (\"0.00000000000000000001\") / 1.0e-20 - 1.0) < 1.0e-10)");
  }
  
  /* Case 53 */
  if (!((fabs (atof ("0.0000000000000000001") / 1.0e-19 - 1.0) < 1.0e-10))) {
    failed ("Case 53: (fabs (atof (\"0.0000000000000000001\") / 1.0e-19 - 1.0) < 1.0e-10)");
  }
  
  /* Case 54 */
  if (!((fabs (atof ("0.000000000000000001") / 1.0e-18 - 1.0) < 1.0e-10))) {
    failed ("Case 54: (fabs (atof (\"0.000000000000000001\") / 1.0e-18 - 1.0) < 1.0e-10)");
  }
  
  /* Case 55 */
  if (!((fabs (atof ("0.00000000000000001") / 1.0e-17 - 1.0) < 1.0e-10))) {
    failed ("Case 55: (fabs (atof (\"0.00000000000000001\") / 1.0e-17 - 1.0) < 1.0e-10)");
  }
  
  /* Case 56 */
  if (!((fabs (atof ("0.0000000000000001") / 1.0e-16 - 1.0) < 1.0e-10))) {
    failed ("Case 56: (fabs (atof (\"0.0000000000000001\") / 1.0e-16 - 1.0) < 1.0e-10)");
  }
  
  /* Case 57 */
  if (!((fabs (atof ("0.000000000000001") / 1.0e-15 - 1.0) < 1.0e-10))) {
    failed ("Case 57: (fabs (atof (\"0.000000000000001\") / 1.0e-15 - 1.0) < 1.0e-10)");
  }
  
  /* Case 58 */
  if (!((fabs (atof ("0.00000000000001") / 1.0e-14 - 1.0) < 1.0e-10))) {
    failed ("Case 58: (fabs (atof (\"0.00000000000001\") / 1.0e-14 - 1.0) < 1.0e-10)");
  }
  
  /* Case 59 */
  if (!((fabs (atof ("0.0000000000001") / 1.0e-13 - 1.0) < 1.0e-10))) {
    failed ("Case 59: (fabs (atof (\"0.0000000000001\") / 1.0e-13 - 1.0) < 1.0e-10)");
  }
  
  /* Case 60 */
  if (!((fabs (atof ("0.000000000001") / 1.0e-12 - 1.0) < 1.0e-10))) {
    failed ("Case 60: (fabs (atof (\"0.000000000001\") / 1.0e-12 - 1.0) < 1.0e-10)");
  }
  
  /* Case 61 */
  if (!((fabs (atof ("0.00000000001") / 1.0e-11 - 1.0) < 1.0e-10))) {
    failed ("Case 61: (fabs (atof (\"0.00000000001\") / 1.0e-11 - 1.0) < 1.0e-10)");
  }
  
  /* Case 62 */
  if (!((fabs (atof ("0.0000000001") / 1.0e-10 - 1.0) < 1.0e-10))) {
    failed ("Case 62: (fabs (atof (\"0.0000000001\") / 1.0e-10 - 1.0) < 1.0e-10)");
  }
  
  /* Case 63 */
  if (!((fabs (atof ("0.000000001") / 1.0e-09 - 1.0) < 1.0e-10))) {
    failed ("Case 63: (fabs (atof (\"0.000000001\") / 1.0e-09 - 1.0) < 1.0e-10)");
  }
  
  /* Case 64 */
  if (!((fabs (atof ("0.00000001") / 1.0e-08 - 1.0) < 1.0e-10))) {
    failed ("Case 64: (fabs (atof (\"0.00000001\") / 1.0e-08 - 1.0) < 1.0e-10)");
  }
  
  /* Case 65 */
  if (!((fabs (atof ("0.0000001") / 1.0e-07 - 1.0) < 1.0e-10))) {
    failed ("Case 65: (fabs (atof (\"0.0000001\") / 1.0e-07 - 1.0) < 1.0e-10)");
  }
  
  /* Case 66 */
  if (!((fabs (atof ("0.000001") / 1.0e-06 - 1.0) < 1.0e-10))) {
    failed ("Case 66: (fabs (atof (\"0.000001\") / 1.0e-06 - 1.0) < 1.0e-10)");
  }
  
  /* Case 67 */
  if (!((fabs (atof ("0.00001") / 1.0e-05 - 1.0) < 1.0e-10))) {
    failed ("Case 67: (fabs (atof (\"0.00001\") / 1.0e-05 - 1.0) < 1.0e-10)");
  }
  
  /* Case 68 */
  if (!((fabs (atof ("0.0001") / 1.0e-04 - 1.0) < 1.0e-10))) {
    failed ("Case 68: (fabs (atof (\"0.0001\") / 1.0e-04 - 1.0) < 1.0e-10)");
  }
  
  /* Case 69 */
  if (!((fabs (atof ("0.001") / 1.0e-03 - 1.0) < 1.0e-10))) {
    failed ("Case 69: (fabs (atof (\"0.001\") / 1.0e-03 - 1.0) < 1.0e-10)");
  }
  
  /* Case 70 */
  if (!((fabs (atof ("0.01") / 1.0e-02 - 1.0) < 1.0e-10))) {
    failed ("Case 70: (fabs (atof (\"0.01\") / 1.0e-02 - 1.0) < 1.0e-10)");
  }
  
  /* Case 71 */
  if (!((fabs (atof ("0.1") / 1.0e-01 - 1.0) < 1.0e-10))) {
    failed ("Case 71: (fabs (atof (\"0.1\") / 1.0e-01 - 1.0) < 1.0e-10)");
  }
  
  /* Case 72 */
  if (!((fabs (atof ("1.0") / 1.0e00 - 1.0) < 1.0e-10))) {
    failed ("Case 72: (fabs (atof (\"1.0\") / 1.0e00 - 1.0) < 1.0e-10)");
  }
  
  /* Case 73 */
  if (!((fabs (atof ("10.0") / 1.0e01 - 1.0) < 1.0e-10))) {
    failed ("Case 73: (fabs (atof (\"10.0\") / 1.0e01 - 1.0) < 1.0e-10)");
  }
  
  /* Case 74 */
  if (!((fabs (atof ("100.0") / 1.0e02 - 1.0) < 1.0e-10))) {
    failed ("Case 74: (fabs (atof (\"100.0\") / 1.0e02 - 1.0) < 1.0e-10)");
  }
  
  /* Case 75 */
  if (!((fabs (atof ("1000.0") / 1.0e03 - 1.0) < 1.0e-10))) {
    failed ("Case 75: (fabs (atof (\"1000.0\") / 1.0e03 - 1.0) < 1.0e-10)");
  }
  
  /* Case 76 */
  if (!((fabs (atof ("10000.0") / 1.0e04 - 1.0) < 1.0e-10))) {
    failed ("Case 76: (fabs (atof (\"10000.0\") / 1.0e04 - 1.0) < 1.0e-10)");
  }
  
  /* Case 77 */
  if (!((fabs (atof ("100000.0") / 1.0e05 - 1.0) < 1.0e-10))) {
    failed ("Case 77: (fabs (atof (\"100000.0\") / 1.0e05 - 1.0) < 1.0e-10)");
  }
  
  /* Case 78 */
  if (!((fabs (atof ("1000000.0") / 1.0e06 - 1.0) < 1.0e-10))) {
    failed ("Case 78: (fabs (atof (\"1000000.0\") / 1.0e06 - 1.0) < 1.0e-10)");
  }
  
  /* Case 79 */
  if (!((fabs (atof ("10000000.0") / 1.0e07 - 1.0) < 1.0e-10))) {
    failed ("Case 79: (fabs (atof (\"10000000.0\") / 1.0e07 - 1.0) < 1.0e-10)");
  }
  
  /* Case 80 */
  if (!((fabs (atof ("100000000.0") / 1.0e08 - 1.0) < 1.0e-10))) {
    failed ("Case 80: (fabs (atof (\"100000000.0\") / 1.0e08 - 1.0) < 1.0e-10)");
  }
  
  /* Case 81 */
  if (!((fabs (atof ("1000000000.0") / 1.0e09 - 1.0) < 1.0e-10))) {
    failed ("Case 81: (fabs (atof (\"1000000000.0\") / 1.0e09 - 1.0) < 1.0e-10)");
  }
  
  /* Case 82 */
  if (!((fabs (atof ("10000000000.0") / 1.0e10 - 1.0) < 1.0e-10))) {
    failed ("Case 82: (fabs (atof (\"10000000000.0\") / 1.0e10 - 1.0) < 1.0e-10)");
  }
  
  /* Case 83 */
  if (!((fabs (atof ("100000000000.0") / 1.0e11 - 1.0) < 1.0e-10))) {
    failed ("Case 83: (fabs (atof (\"100000000000.0\") / 1.0e11 - 1.0) < 1.0e-10)");
  }
  
  /* Case 84 */
  if (!((fabs (atof ("1000000000000.0") / 1.0e12 - 1.0) < 1.0e-10))) {
    failed ("Case 84: (fabs (atof (\"1000000000000.0\") / 1.0e12 - 1.0) < 1.0e-10)");
  }
  
  /* Case 85 */
  if (!((fabs (atof ("10000000000000.0") / 1.0e13 - 1.0) < 1.0e-10))) {
    failed ("Case 85: (fabs (atof (\"10000000000000.0\") / 1.0e13 - 1.0) < 1.0e-10)");
  }
  
  /* Case 86 */
  if (!((fabs (atof ("100000000000000.0") / 1.0e14 - 1.0) < 1.0e-10))) {
    failed ("Case 86: (fabs (atof (\"100000000000000.0\") / 1.0e14 - 1.0) < 1.0e-10)");
  }
  
  /* Case 87 */
  if (!((fabs (atof ("1000000000000000.0") / 1.0e15 - 1.0) < 1.0e-10))) {
    failed ("Case 87: (fabs (atof (\"1000000000000000.0\") / 1.0e15 - 1.0) < 1.0e-10)");
  }
  
  /* Case 88 */
  if (!((fabs (atof ("10000000000000000.0") / 1.0e16 - 1.0) < 1.0e-10))) {
    failed ("Case 88: (fabs (atof (\"10000000000000000.0\") / 1.0e16 - 1.0) < 1.0e-10)");
  }
  
  /* Case 89 */
  if (!((fabs (atof ("100000000000000000.0") / 1.0e17 - 1.0) < 1.0e-10))) {
    failed ("Case 89: (fabs (atof (\"100000000000000000.0\") / 1.0e17 - 1.0) < 1.0e-10)");
  }
  
  /* Case 90 */
  if (!((fabs (atof ("1000000000000000000.0") / 1.0e18 - 1.0) < 1.0e-10))) {
    failed ("Case 90: (fabs (atof (\"1000000000000000000.0\") / 1.0e18 - 1.0) < 1.0e-10)");
  }
  
  /* Case 91 */
  if (!((fabs (atof ("10000000000000000000.0") / 1.0e19 - 1.0) < 1.0e-10))) {
    failed ("Case 91: (fabs (atof (\"10000000000000000000.0\") / 1.0e19 - 1.0) < 1.0e-10)");
  }
  
  /* Case 92 */
  if (!((fabs (atof ("100000000000000000000.0") / 1.0e20 - 1.0) < 1.0e-10))) {
    failed ("Case 92: (fabs (atof (\"100000000000000000000.0\") / 1.0e20 - 1.0) < 1.0e-10)");
  }
  
  /* Case 93 */
  if (!((fabs (atof ("1000000000000000000000.0") / 1.0e21 - 1.0) < 1.0e-10))) {
    failed ("Case 93: (fabs (atof (\"1000000000000000000000.0\") / 1.0e21 - 1.0) < 1.0e-10)");
  }
  
  /* Case 94 */
  if (!((fabs (atof ("10000000000000000000000.0") / 1.0e22 - 1.0) < 1.0e-10))) {
    failed ("Case 94: (fabs (atof (\"10000000000000000000000.0\") / 1.0e22 - 1.0) < 1.0e-10)");
  }
  
  /* Case 95 */
  if (!((fabs (atof ("100000000000000000000000.0") / 1.0e23 - 1.0) < 1.0e-10))) {
    failed ("Case 95: (fabs (atof (\"100000000000000000000000.0\") / 1.0e23 - 1.0) < 1.0e-10)");
  }
  
  /* Case 96 */
  if (!((fabs (atof ("1000000000000000000000000.0") / 1.0e24 - 1.0) < 1.0e-10))) {
    failed ("Case 96: (fabs (atof (\"1000000000000000000000000.0\") / 1.0e24 - 1.0) < 1.0e-10)");
  }
  
  /* Case 97 */
  if (!((fabs (atof ("10000000000000000000000000.0") / 1.0e25 - 1.0) < 1.0e-10))) {
    failed ("Case 97: (fabs (atof (\"10000000000000000000000000.0\") / 1.0e25 - 1.0) < 1.0e-10)");
  }
  
  /* Case 98 */
  if (!((fabs (atof ("100000000000000000000000000.0") / 1.0e26 - 1.0) < 1.0e-10))) {
    failed ("Case 98: (fabs (atof (\"100000000000000000000000000.0\") / 1.0e26 - 1.0) < 1.0e-10)");
  }
  
  /* Case 99 */
  if (!((fabs (atof ("1000000000000000000000000000.0") / 1.0e27 - 1.0) < 1.0e-10))) {
    failed ("Case 99: (fabs (atof (\"1000000000000000000000000000.0\") / 1.0e27 - 1.0) < 1.0e-10)");
  }
  
  /* Case 100 */
  if (!((fabs (atof ("10000000000000000000000000000.0") / 1.0e28 - 1.0) < 1.0e-10))) {
    failed ("Case 100: (fabs (atof (\"10000000000000000000000000000.0\") / 1.0e28 - 1.0) < 1.0e-10)");
  }
  
  /* Case 101 */
  if (!((fabs (atof ("100000000000000000000000000000.0") / 1.0e29 - 1.0) < 1.0e-10))) {
    failed ("Case 101: (fabs (atof (\"100000000000000000000000000000.0\") / 1.0e29 - 1.0) < 1.0e-10)");
  }
  
  /* Case 102 */
  if (!((fabs (atof ("1000000000000000000000000000000.0") / 1.0e30 - 1.0) < 1.0e-10))) {
    failed ("Case 102: (fabs (atof (\"1000000000000000000000000000000.0\") / 1.0e30 - 1.0) < 1.0e-10)");
  }
  
  /* Case 103 */
  if (!((fabs (atof ("10000000000000000000000000000000.0") / 1.0e31 - 1.0) < 1.0e-10))) {
    failed ("Case 103: (fabs (atof (\"10000000000000000000000000000000.0\") / 1.0e31 - 1.0) < 1.0e-10)");
  }
  
  /* Case 104 */
  if (!((fabs (atof ("100000000000000000000000000000000.0") / 1.0e32 - 1.0) < 1.0e-10))) {
    failed ("Case 104: (fabs (atof (\"100000000000000000000000000000000.0\") / 1.0e32 - 1.0) < 1.0e-10)");
  }
  
  /* Case 105 */
  if (!((fabs (atof ("1000000000000000000000000000000000.0") / 1.0e33 - 1.0) < 1.0e-10))) {
    failed ("Case 105: (fabs (atof (\"1000000000000000000000000000000000.0\") / 1.0e33 - 1.0) < 1.0e-10)");
  }
  
  /* Case 106 */
  if (!((fabs (atof ("10000000000000000000000000000000000.0") / 1.0e34 - 1.0) < 1.0e-10))) {
    failed ("Case 106: (fabs (atof (\"10000000000000000000000000000000000.0\") / 1.0e34 - 1.0) < 1.0e-10)");
  }
  
  /* Case 107 */
  if (!((fabs (atof ("100000000000000000000000000000000000.0") / 1.0e35 - 1.0) < 1.0e-10))) {
    failed ("Case 107: (fabs (atof (\"100000000000000000000000000000000000.0\") / 1.0e35 - 1.0) < 1.0e-10)");
  }
  
  /* Case 108 */
  if (!((fabs (atof ("1000000000000000000000000000000000000.0") / 1.0e36 - 1.0) < 1.0e-10))) {
    failed ("Case 108: (fabs (atof (\"1000000000000000000000000000000000000.0\") / 1.0e36 - 1.0) < 1.0e-10)");
  }
  
  /* Case 109 */
  if (!((fabs (atof ("10000000000000000000000000000000000000.0") / 1.0e37 - 1.0) < 1.0e-10))) {
    failed ("Case 109: (fabs (atof (\"10000000000000000000000000000000000000.0\") / 1.0e37 - 1.0) < 1.0e-10)");
  }
  
  /* Case 110 */
  if (!((fabs (atof ("100000000000000000000000000000000000000.0") / 1.0e38 - 1.0) < 1.0e-10))) {
    failed ("Case 110: (fabs (atof (\"100000000000000000000000000000000000000.0\") / 1.0e38 - 1.0) < 1.0e-10)");
  }
  
}


void main ()
{
  test ("test-atof", "Test for ANSI Standard C: 7.10.1.1");

  do_test ();

  result ();
}

